# the following steps assume you have run the script "Sample Dispro Calculations.R". Do so now if you have not already

# Step 6: specify the location of the sample rmd file provided
RmdFile <- "C:/File/Location/of/Sample_Racial_Equity_Report_2020.Rmd"

# Step 7: Install TinyTex (if you haven't already)
# while any LaTeX distribution can technically work, tinytex is recommended as it automatically installs required LaTeX packages for you.
# unistall other tex distributions before installing tinytex
# for troubleshooting, see https://yihui.org/tinytex/
install.packages('tinytex')
tinytex::install_tinytex()

# Step 8: Install other required packages (if not already installed)
install.packages(c('knitr', 'kableExtra'))

library(knitr); library(tidyverse); library(kableExtra)

# Step 9: Run the remainder of the script! the sample reports will output to the directory you specified in step 1.

# The master file of all calculations
DisproMaster <- read.csv(paste0(ReportingYear, " Sample DisproMaster.csv"), stringsAsFactors = F)
# The file of all group-level racial dispro determinations and district info (district name, etc)
DisproDeterms <- read.csv(paste0(ReportingYear, " Sample DisproDeterms.csv"), stringsAsFactors = F)
# The file of all racial dispro determinations that meet state's disproportionality criteria
DisproDistricts <- read.csv(paste0(ReportingYear, " Sample DisproDistricts.csv"), stringsAsFactors = F)

####################
##### packages #####
####################
library(knitr); library(tidyverse); library(readxl); library(kableExtra)
options(knitr.table.format = "latex")

# fill in any missing combinations of race/group/year in master file
tmp <- tibble()
for (i in c('ID', 'ENV', 'DSC')) {
  tmp2 <- subset(DisproMaster, CATEGORY_CODE==i)  %>% complete(DCODE, CATEGORY_CODE, nesting(GROUP_CODE, RACE, REPORTING_YEAR))
  tmp <- bind_rows(tmp, tmp2)
}
# round RR to nearest 1/1000
DisproMaster$RegAltRatio <- as.character(round(DisproMaster$RegAltRatio, 4))

# convert numeric data elements to characters, so NAs will be more readable by text to speech.
DisproDeterms$YearsCell <- as.character(DisproDeterms$YearsCell)
DisproDeterms$YearsProgress <- as.character(DisproDeterms$YearsProgress)
DisproDeterms$Dispro <- as.character(DisproDeterms$Dispro)

###################################
##### functions and shortcuts #####
###################################
# LaTeX checkboxes
checkbox <- "\\Large\\makebox[0pt][l]{$\\square$}\\raisebox{.15ex}{\\hspace{0.1em}$\\checkmark$}\\normalsize"
emptybox <- "\\Large$\\square$\\normalsize"

# easy recode vectors
RECODE_CATEGORY <- c(ID = "Identification", ENV = "Placement", DSC = "Discipline")
RECODE_RACE <- c(A = "Asian", B = "Black", H = "Hispanic/Latinx", I = "American Indian", P = "Pacific Islander", `T` = "Multi Race", W = "White", `@UNK` = 'Unknown')
RECODE_GROUP_ID <- c(A = 'Autism', ALL = 'Students w/ IEPs', EBD = 'Emotional Behavioral', ID = 'Intellectual Disability',
                     LD = 'Specific Learning Disability', OHI = 'Other Health Impairment', SL = 'Speech \\& Language')
RECODE_GROUP_ENV <- c(Ind5b = "Less Than 40\\% Regular Class", Ind5c = 'Separate School (ages 6-21)',
                      Ind6b = 'Separate School (ages 3-5)')
RECODE_GROUP_DSC <- c(ALL = 'All Disciplinary Removals', ISGT = "In-School > 10 Days", ISLE = "In-School <= 10 Days",
                      OSGT = "Out-of-School > 10 Days", OSLE = "Out-of-School <= 10 Days")

# function to create columns if they don't exist
fncols <- function(data, colname) {
  add <-colname[!colname %in% names(data)]
  if(length(add)!=0) data[add] <- NA
  data
}

# custom table function
CatTable <- function(category, groups, title, recode.values, FontSize = 12) {
  tmp <- as_tibble(merge(subset(SubsetDeterms, CATEGORY_CODE == category & GROUP_CODE %in% groups,
                                select = c(CATEGORY_CODE, GROUP_CODE, RACE, YearsCell, YearsProgress, Dispro)),
                         spread(subset(SubsetMaster, CATEGORY_CODE == category & GROUP_CODE %in% groups,
                                       select = c(GROUP_CODE, RACE, REPORTING_YEAR, RegAltRatio)),
                                REPORTING_YEAR, RegAltRatio),
                         all = T))
  if (dim(tmp)[1] == 0) {
    tmp <- tibble(GROUP_CODE = NA, CATEGORY_CODE = NA, RACE = NA, YearsCell = NA, YearsProgress = NA, Dispro = NA)
  }
  
  tmp <- if(category=='DSC') {
    fncols(tmp, c(as.character(ReportingYear-2), as.character(ReportingYear-1), as.character(ReportingYear)))
  } else {
    fncols(tmp, c(as.character(ReportingYear-1), as.character(ReportingYear), as.character(ReportingYear+1)))
  }
  
  tmp <- tmp[order(tmp$GROUP_CODE),]
  
  tmp$GROUP_CODE <- recode(tmp$GROUP_CODE, !!!recode.values)
  tmp$RACE <- recode(tmp$RACE, !!!RECODE_RACE)
  
  select(tmp, -c(CATEGORY_CODE, Dispro)) %>% 
    rename(Group = GROUP_CODE,
           Race = RACE,
           `Met Cell Size` = YearsCell,
           `Made Progress` = YearsProgress) %>%
    kable(escape = F, caption = title, booktabs = T, longtable = T, align = 'c', linesep = "") %>%
    collapse_rows(columns = 1, latex_hline = "major", valign = "middle") %>%
    row_spec(which(tmp$YearsCell >= 2 & tmp[,5] >= 2 & tmp[,6] >= 2 & tmp$RACE != 'W'), bold = T, italic = T) %>%
    row_spec(which(as.logical(tmp$Dispro) == T & tmp$RACE != 'W'), bold = T, italic = T, color = "black", background = "#99CA3C") %>%
    add_header_above(c(" " = 2, "Years (Count)" = 2, "Risk Ratio" = 3)) %>%
    kable_styling(latex_options = c("hold_position", "repeat_header"), font_size = FontSize) %>%
    footnote("Where applicable, records that meet the criteria for disproportionality are highlighted green. Bold but unhighlighted records are not yet disproportionate, but may meet the criteria for disproportionality next year.", threeparttable = T)
}

# Remove special characters from district names, as they may cause errors in report generation
DisproDeterms$DNAME <- gsub("&", "and",  DisproDeterms$DNAME, ignore.case = FALSE)
DisproDeterms$DNAME <- gsub("#", "",  DisproDeterms$DNAME, ignore.case = FALSE)
DisproDeterms$DNAME <- gsub("/", "_", DisproDeterms$DNAME, ignore.case = FALSE)

# Generating naming conventions for files and removing meta characters
directory <- subset(DisproDeterms, !duplicated(DCODE), select = c(DCODE, DNAME))
directory$DNAME <- gsub(" ", "_", directory$DNAME, ignore.case = FALSE)
directory$DNAME <- gsub("-", "_", directory$DNAME, ignore.case = FALSE)
directory$DNAME <- gsub("/", "_", directory$DNAME, ignore.case = FALSE)
directory$DNAME <- substr(directory$DNAME, 1, 25) # keeps the names short
directory$DCODE <- as.numeric(directory$DCODE)

# create the list of districts for whom you are generating reports
DCODE_LIST <- directory$DCODE

## a failed reports object that saves info about any reports that fail
failed <- data.frame()
out <- NULL

#############################
##### Report Generation #####
#############################

## Loop to make the report 
for (i in 1:length(DCODE_LIST)){
  print(paste0("Writing Report ", i, " of ", length(DCODE_LIST), ": ", DCODE_LIST[i], " at ", Sys.time()))
  dc <- DCODE_LIST[i]
  SubsetMaster <- subset(DisproMaster, DCODE == dc)
  SubsetDeterms <- subset(DisproDeterms, DCODE == dc)
  SubsetDistricts <- subset(DisproDistricts, DCODE == dc)
  SubsetLEAdeterms <- subset(LEAdeterms, DC == dc)
  dname <- directory$DNAME[directory$DCODE == dc]
  
  # this prints each report to their own directory (used for SAFE uploads)
  out <- try(rmarkdown::render(RmdFile, # this try() statement lets the loop keep running when it fails on a given record; it just moves on to the next
                               output_format="pdf_document",
                               output_file=paste0("Racial_Equity_", ReportingYear, "_", dc, "_", dname, ".pdf"),
                               quiet = TRUE)) # This argument prevents output from filling up the console
  if(class(out) == "try-error") {
    failed <- rbind(failed, data.frame(DCODE=dc, DNAME=directory$DNAME[directory$DCODE == dc], error= as.character(out)))
  }
}